# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from command import Command

import os
from django.template import Template, Context
from django.conf import settings as django_settings
from ionas.request import Entry, EntryList, RequestData, XmlNode, XmlConfNode
from fal_info import Results
from iobase.cmdexec import (CmdExec, PIPE, NULL)
from datetime import time
import re
from copy import deepcopy
import shutil

CONF_PATH = "/mnt/data/xmlconf/config-syslogsrv.xml"
SYSLOGNG_TEMPLATE_PATH = "/mnt/data/conf/syslogsrv/templates/syslog-ng-extend.conf.tpl"
SYSLOGNG_DEPLOY_PATH = "/mnt/data/etc/syslog-ng/conf.d/syslog-ng-extend.conf"

AUTOREMOVE_TEMPLATE_PATH = "/mnt/data/conf/syslogsrv/templates/autoremove.py.tpl"
AUTOREMOVE_DEPLOY_PATH = "/mnt/data/conf/syslogsrv/autoremove.py"

LOGROTATE_TEMPLATE_PATH = "/mnt/data/conf/syslogsrv/templates/syslogsrv.logrotate.tpl"
LOGROTATE_TEMPORARY_PATH = "/mnt/data/conf/syslogsrv/temporary/syslogsrv.logrotate.tmp"
LOGROTATE_DEPLOY_PATH = "/mnt/data/conf/syslogsrv/syslogsrv.logrotate"

FACILITIES = (u"",
              u"kern"        , u"user"    , u"mail"   ,
              u"daemon"      , u"auth"    , u"syslog" ,
              u"lpr"         , u"news"    , u"uucp"   ,
              u"cron"        , u"authpriv", u"ftp"    ,
              u"ntp"         , u"security", u"console",
              u"solaris-cron", u"local0"  , u"local1" ,
              u"local2"      , u"local3"  , u"local4" ,
              u"local5"      , u"local6"  , u"local7" ,)

PRIORITIES = (u"",
              u"emerg"  , u"alert" , u"crit", u"err", 
              u"warning", u"notice", u"info", u"debug")

ROTATE_DEFAULT_SIZE = "100"


if not django_settings.configured:
    django_settings.configure()

class FALSyslogSrv(Command):

    def load(self):

        def __dict(entry):
            d = {}
            try:
                syslog_node = entry.get()

                try:
                    d["settings"] = {}
                    d["settings"]["enabled"] = syslog_node.get_node("settings").is_enabled()
                    d["settings"]["udp"] = syslog_node.get_value("settings/udp")
                    d["settings"]["tcp"] = syslog_node.get_value("settings/tcp")
                    d["settings"]["size"] = syslog_node.get_value("settings/size")
                    d["settings"]["autodelete"] = {}
                    d["settings"]["autodelete"]["enabled"] = syslog_node.get_node("settings/autodelete").is_enabled()
                    d["settings"]["autodelete"]["expire"] = {}
                    d["settings"]["autodelete"]["expire"]["days"] = syslog_node.get_value("settings/autodelete/expire/days")
                    d["settings"]["autodelete"]["expire"]["percent"] = syslog_node.get_value("settings/autodelete/expire/percent")
                except:
                    d["settings"] = {}
                    d["settings"]["enabled"] = None
                    d["settings"]["udp"] = 514
                    d["settings"]["tcp"] = 601
                    d["settings"]["size"] = ROTATE_DEFAULT_SIZE
                    d["settings"]["autodelete"] = {}
                    d["settings"]["autodelete"]["enabled"] = None
                    d["settings"]["autodelete"]["expire"] = {}
                    d["settings"]["autodelete"]["expire"]["days"] = None
                    d["settings"]["autodelete"]["expire"]["percent"] = None

                try:
                    d["alertlist"] = []
                    for alert_node in syslog_node.get_node_list("alertlist/alert"):
                        adict = {
                            "name": alert_node.get_value("name"),
                            "hostname": alert_node.get_value("hostname"),
                            "message": alert_node.get_value("message"),
                            "facilities": alert_node.get_value("facilities"),
                            "priorities": alert_node.get_value("priorities"),
                        }
                        d["alertlist"].append(adict)
                except:
                    d["alertlist"] = []

            except:
                pass
            return d

        try:
            entry = self._load()
            return __dict(entry)
        except:
            return {}

    def _load(self):
        entry = Entry("load")
        if os.path.exists(CONF_PATH):
            with open(CONF_PATH) as fp:
                txt = fp.read()
                xml = XmlNode(txt)
                entry.set(xml)
        return entry

    def apply(self, data=None, log=True, messages={}):
        __default_messages = {
            Results.OK: ["MSG_SYSLOG_CHANGE", {}],
            None: ["ERR_SYSLOG_CHANGE", {}],
        }
        __default_messages.update(messages)

        try:
            entry = self._apply(data=data)
            result = getattr(entry, "get_result", lambda: None)()
        except:
            result = None

        message = __default_messages.get(result, __default_messages[None])
        self._set_message(message[0], arglist=message[1], log=log)

        return result in [Results.OK]

    def _apply(self, data=None):
        entry = Entry("apply", """<syslogsrv/>""")
        
        _dict = self.load()
        if data:
            _dict.update(data)
        
        data = _dict
        
        try:
            # apply syslog-ng.conf
            self._apply_syslogng_conf(data)
            
            # apply autoremove.sh
            self._apply_autoremove(data)
            
            # restart syslog-ng
            self._restart_syslogng()
            
            # apply logrotate
            self._apply_logrotate(data)
            
            # save xmlconf
            self._save_xmlconf(data, entry)
            
            # set result
            entry.set_result(Results.OK)
        except Exception as ex:
            pass
        
        return entry
        
    def _save_xmlconf(self, data, entry):
        syslog_node = entry.get()
        try:
            syslog_node.append_child("settings")
            settings_node = syslog_node.get_node("settings")
            settings_node.enable(data["settings"]["enabled"])
            settings_node.append_child("udp", text=str(data["settings"]["udp"]))
            settings_node.append_child("tcp", text=str(data["settings"]["tcp"]))
            settings_node.append_child("size", text=str(data["settings"]["size"]))
            settings_node.append_child("autodelete")
            autodelete_node = settings_node.get_node("autodelete")
            autodelete_node.enable(data["settings"]["autodelete"]["enabled"])
            autodelete_node.append_child("expire")
            expire_node = autodelete_node.get_node("expire")
            expire_node.append_child("days", text=str(data["settings"]["autodelete"]["expire"]["days"]))
            expire_node.append_child("percent", text=str(data["settings"]["autodelete"]["expire"]["percent"]))
        except:
            pass

        try:
            syslog_node.append_child("alertlist")
            alertlist_node = syslog_node.get_node("alertlist")
            for alert in data["alertlist"]:
                alert_node = XmlConfNode("<alert/>")
                alert_node.append_child("name", text=alert["name"])
                alert_node.append_child("hostname", text=alert["hostname"])
                alert_node.append_child("message", text=alert["message"])
                alert_node.append_child("facilities", text=alert["facilities"])
                alert_node.append_child("priorities", text=alert["priorities"])
                alertlist_node.append_child(alert_node)
        except:
            pass

        return self._save(entry)

    def _save(self, entry):
        xml = str(entry.get())
        with open(CONF_PATH, "w") as fp:
            fp.write(xml)
        return entry

    def _apply_syslogng_conf(self, data):
        _data = deepcopy(data)
        
        if "alertlist" in data:
            _data["alertlist"] = []
            for alert in data["alertlist"]:
                # build filter
                filters = []
                if alert["hostname"]:
                    filters.append('host("{0}")'.format(self.escape_syslogng_conf(alert["hostname"])))
                if alert["message"]:
                    filters.append('message("{0}")'.format(self.escape_syslogng_conf(alert["message"])))
                if alert["facilities"]:
                    filters.append('facility({0})'.format(alert["facilities"]))
                if alert["priorities"]:
                    filters.append('priority({0})'.format(alert["priorities"]))
                
                # build alert
                if len(filters) > 0:
                    item = {}
                    item["filter"] = ' and '.join(filters) + ";"
                    item["name"] = alert["name"]
                    _data["alertlist"].append(item)
        
        # build conf
        context = Context(_data)
        with open(SYSLOGNG_TEMPLATE_PATH) as fp:
            tpl_src = fp.read()
            tpl = Template(tpl_src)
            dest = tpl.render(context)
        
        # deploy conf
        with open(SYSLOGNG_DEPLOY_PATH, "w") as fp:
            fp.write(dest)
    
    def escape_syslogng_conf(self, txt):
        pat = r'(["\\])'
        rep = r'\\\1'
        return re.sub(pat, rep, txt)
    
    def _apply_autoremove(self, data):
        # build conf
        _data = {}
        if data["settings"]["autodelete"]["enabled"]:
            _data["enabled"] = ""
        else:
            _data["enabled"] = "return"
        _data["percent"] = data["settings"]["autodelete"]["expire"]["percent"]
        _data["days"] = data["settings"]["autodelete"]["expire"]["days"]
        
        context = Context(_data)
        with open(AUTOREMOVE_TEMPLATE_PATH) as fp:
            tpl_src = fp.read()
            tpl = Template(tpl_src)
            dest = tpl.render(context)
        
        # deploy conf
        with open(AUTOREMOVE_DEPLOY_PATH, "w") as fp:
            fp.write(dest)

    def _restart_syslogng(self):
        cmd = ["/usr/bin/sudo", "/usr/sbin/service", "syslog-ng", "restart"]
        CmdExec(cmd, stdin=NULL, stdout=NULL, stderr=NULL,
            raise_exc=True, close_fds=True).execute()

    def _apply_logrotate(self, data):
        # build conf
        try:
            rotate_size = int(data["settings"]["size"])
        except:
            rotate_size = int(ROTATE_DEFAULT_SIZE)

        context = Context({"size": "%dM" % (rotate_size,)})
        with open(LOGROTATE_TEMPLATE_PATH) as fp:
            tpl_src = fp.read()
            tpl = Template(tpl_src)
            dest = tpl.render(context)
        
        # generate conf
        with open(LOGROTATE_TEMPORARY_PATH, "w") as fp:
            fp.write(dest)

        # deploy conf
        cmd = ["/usr/bin/sudo", "cp", LOGROTATE_TEMPORARY_PATH, LOGROTATE_DEPLOY_PATH]
        CmdExec(cmd, stdin=NULL, stdout=NULL, stderr=NULL,
            raise_exc=True, close_fds=True).execute()


if __name__ == u"__main__":
    fal = FALSyslogSrv()
