# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from django import forms
from django.forms.formsets import formset_factory
from django.forms import IntegerField
from django.forms import DateField
from django.forms import DateTimeField
from django.forms import CharField
from django.core.exceptions import ValidationError
from django.utils.translation import ugettext as _
from django.utils import timezone
from datetime import datetime, timedelta, tzinfo
import os
import glob
import re

class LogFilter:
    LOG_PATH = '/mnt/sataraid1/share/LAN DISK Log/SysLogServer/%s/*/*'
    LOG_DATE = '[0-9]{8}'
    LOG_FILE = '^syslog_' + LOG_DATE + '.*.log.*$'
    LOG_MAX  = 1000
    BIT_OTHERS = 0x80000000

    PRIORITIES = (
        u"emerg"  , u"alert" , u"crit", u"err", 
        u"warning", u"notice", u"info", u"debug")

    FACILITIES = (
        u"kern"        , u"user"    , u"mail"   ,
        u"daemon"      , u"auth"    , u"syslog" ,
        u"lpr"         , u"news"    , u"uucp"   ,
        u"cron"        , u"authpriv", u"ftp"    ,
        u"ntp"         , u"security", u"console",
        u"solaris-cron", u"local0"  , u"local1" ,
        u"local2"      , u"local3"  , u"local4" ,
        u"local5"      , u"local6"  , u"local7" ,)

    def str_to_datetime(self, str):
        try:
            e = str.split('T')
            d = e[0].split('-')
            e = e[1].split('.')
            t = e[0].split('+')
            t = t[0].split(':')
            # e[1] に残りの microsecond と timezoneがある
            return datetime(int(d[0]), int(d[1]), int(d[2]),
                int(t[0]), int(t[1]), int(t[2]), 
                microsecond=0, tzinfo=self.tz)
        except:
            pass
        return None

    def _is_target_file(self, date):
       if len(date) != 8:
           return False
       if self.datetime_from:
           dt = datetime(int(date[0:4]), int(date[4:6]), int(date[6:8]),
               23, 59, 59, tzinfo=self.tz)
           if self.datetime_from > dt:
              return False;
       if self.datetime_to:
           dt = datetime(int(date[0:4]), int(date[4:6]), int(date[6:8]),
               0, 0, 0, tzinfo=self.tz)
           if self.datetime_to < dt:
              return False;
       return True

    def _is_matched_datetime(self, date):
       dt = self.str_to_datetime(date)
       if not dt:
           return False
       if self.datetime_from:
           if self.datetime_from > dt:
              return False;
       if self.datetime_to:
           if self.datetime_to < dt:
              return False;
       return True

    def _is_matched_priority(self, priority):
        pris = LogFilter.PRIORITIES
        for p in pris:
            if p == priority:
                return (self.priority_mask & (1 << pris.index(p)) != 0)
        return ((self.priority_mask & LogFilter.OTHERS) != 0)

    def _is_matched_facility(self, facility):
        facs = LogFilter.FACILITIES
        for f in facs:
            if f == facility:
                return (self.facility_mask & (1 << facs.index(f)) != 0)
        return ((self.facility_mask & LogFilter.OTHERS) != 0)

    def _is_matched_keyword(self, content):
        if self.keyword == "":
            return True
        try:
            m = re.search(self.keyword, content)
        except:
            m = None
        return True if m else False

    def _listup_files(self):
        host = self.hostname
        if host == "":
            host = '*'
        files = []
        list = glob.glob(LogFilter.LOG_PATH % host)
        repatter = re.compile(LogFilter.LOG_FILE)
        for fpath in list:
            fname = os.path.basename(fpath)
            match = repatter.match(fname)
            if match:
                fday = re.search('[0-9]{8}', fname)
                if self._is_target_file(fday.group()):
                    files.append(fname + '/' + os.path.dirname(fpath))
        files.sort()
        result = []
        for f in files:
            t = f.split('/', 1)
            result.append(t[1] + '/' + t[0])
        return result

    def _get_date_from_dir(self, path):
        try:
            dir = os.path.dirname(path)
            dirs = dir.split('/')
            dir = dirs[len(dirs) - 1]
            year = int(dir[0:4])
            month = int(dir[4:6])
            return (datetime(year, month, 1, 0, 0, 0, tzinfo=self.tz))
        except:
            pass
        return None

    def _get_date_from_filename(self, path):
        try:
            fname = os.path.basename(path)
            fday = re.search('[0-9]{8}', fname)
            if fday.group():
                year = int(d[0:4])
                month = int(d[4:6])
                day = int(d[6:8])
                return (datetime(year, month, day,
                      0, 0, 0, 0, tzinfo=self.tz))
        except:
            pass
        return None

    def _convert_log(self, path, line):
        dt = self._get_date_from_dir(path)
        #dt= self._get_date_from_filenamedir(path)
        if not dt:
            return None
        p = re.split(' +', line, 4)
        if len(p) < 5:
            return None
        t = p[2].split(':')
        if len(t) != 3:
            return None
        try:
            dt = dt.replace(day = int(p[1]),
                hour = int(t[0]), minute = int(t[1]), second = int(t[2]))
            return("%s %s %06d %s" %
                (dt.isoformat(), p[3], len(self.logs), p[4]))
        except:
            pass
        return None

    def _read_log(self, path):
        with open(path) as f:
            for src in f:
                if len(self.logs) >= LogFilter.LOG_MAX:
                    return;
                line = self._convert_log(path, src)
                if not line:
                    continue
                t = line.split(' ', 5)
                if len(t) < 6:
                    continue
                if not self._is_matched_datetime(t[0]):
                    continue
                if not self._is_matched_priority(t[4]):
                    continue
                if not self._is_matched_facility(t[3]):
                    continue
                if not self._is_matched_keyword(t[5]):
                    continue
                self.logs.append(line.rstrip('\n'))

    def _check_params(self, params):
        self.datetime_from = params.get('datetime_from', None)
        self.datetime_to   = params.get('datetime_to',   None)
        self.hostname      = params.get('hostname',      "*")
        self.keyword       = params.get('keyword',       "")
        self.priority_mask = params.get('priority_mask', 0xFF)
        self.facility_mask = params.get('facility_mask', 0x0FFFFFFF)

    def __init__(self):
        try:
            self.tz = timezone.get_default_timezone()
        except:
            self.tz = JST()

    def collect(self, params):
        self.logs = []
        self._check_params(params)
        files = self._listup_files()
        for path in files:
            if len(self.logs) >= LogFilter.LOG_MAX:
               break
            self._read_log(path)
        self.logs.sort()
        return self.logs

class JST(tzinfo):
    def utcoffset(self, dt):
        return timedelta(hours=9)

    def dst(self, dt):
        return timedelta(0)

    def tzname(self, dt):
        return 'JST'

if __name__ == '__main__':
    lf = LogFilter()
    logs = lf.collect({
        'datetime_from': datetime(2019, 01, 21, 5, 0, 0, 0,tzinfo=lf.tz),
        'datetime_to'  : datetime.now(tz=lf.tz)-timedelta(days=0),
        'hostname'     : "",
        'keyword'      : ".*",
        'priority_mask': 0xFF,
        'facility_mask': 0xFFFFFFFF})
    for log in logs:
        print(log)
    print("log counts = %d" % len(logs))
